package net.lensfrex.oj.collector;

import com.google.gson.*;
import net.lensfrex.oj.collector.data.QuestionDetail;
import net.lensfrex.oj.collector.utils.IOUtil;
import net.lensfrex.oj.collector.utils.NetworkUtil;
import net.lensfrex.oj.collector.utils.Random;
import org.apache.http.client.utils.URIBuilder;

import java.io.IOException;
import java.net.URISyntaxException;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.HashMap;

public class Collector {

    private static final String API_BASE = "https://leetcode.cn/graphql/";

    private static final String DETAILS_REQUEST_BODY = IOUtil.inputStreamToString(Record.class.getResourceAsStream("/details_request_body.txt"), StandardCharsets.UTF_8);

    private static final String INDEX_REQUEST_BODY = IOUtil.inputStreamToString(Record.class.getResourceAsStream("/index_request_body.txt"), StandardCharsets.UTF_8);

    private static final int PAGE_LIMIT = 100;

    private String generatePageRequestUrl(String baseUrl, int offset, int limit, int page) throws URISyntaxException {
        URIBuilder uriBuilder = new URIBuilder(baseUrl);
        uriBuilder.addParameter("paging", "true");
        uriBuilder.addParameter("offset", String.valueOf(offset));
        uriBuilder.addParameter("limit", String.valueOf(limit));
        uriBuilder.addParameter("page", String.valueOf(page));

        return uriBuilder.toString();
    }

    private HashMap<String, String> getHeaders() {
        HashMap<String, String> headers = new HashMap<>();

        headers.put("User-Agent", "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.5060.66 Safari/537.36 Edg/103.0.1264.44");
        headers.put("Host", "leetcode.cn");
        headers.put("Origin", "https://leetcode.cn");
        headers.put("Accept-Encoding", "gzip, deflate, br");
        headers.put("Content-Type", "application/json");

        return headers;
    }

    //
//    private QuestionDetail[] getResults(String url, Map<String, String> headers) throws IOException {
//        String json = NetworkUtil.get(url, headers);
//
//        JsonArray resultJsonArray = JsonParser.parseString(json).getAsJsonObject()
//                .getAsJsonObject("data")
//                .getAsJsonArray("results");
//
//        return new Gson().fromJson(resultJsonArray, QuestionDetail[].class);
//    }

    private final Record record = new Record();
    private static final Random random = new Random();

    // 起始位置
    private static final int START_AT = 0;

    public ArrayList<QuestionDetail> collectAllQuestion() {
        ArrayList<QuestionDetail> QuestionDetails = new ArrayList<>();
        try {
            HashMap<String, String> headers = getHeaders();

            String json = NetworkUtil.post(API_BASE, String.format(INDEX_REQUEST_BODY, 0, 1), headers);

            int totalResult = JsonParser.parseString(json).getAsJsonObject()
                    .getAsJsonObject("data")
                    .getAsJsonObject("problemsetQuestionList")
                    .getAsJsonPrimitive("total").getAsInt();

            int page = (totalResult / PAGE_LIMIT) + (totalResult % PAGE_LIMIT == 0 ? 0 : 1);

            ArrayList<String> questionNames = new ArrayList<>(totalResult);

            System.out.println("Total results: " + totalResult);
            System.out.println("Page Limit: " + PAGE_LIMIT);
            System.out.println("Total pages: " + page);

            // Fetch all question list
            for (int i = 1; i <= page; i++) {
                int offset = (i - 1) * PAGE_LIMIT;
                System.out.println("\n------------------------------------------------------");
                System.out.println("Getting page " + i);
                System.out.println("Offset " + offset);

                json = NetworkUtil.post(API_BASE, String.format(INDEX_REQUEST_BODY, START_AT + offset, PAGE_LIMIT), headers);

                JsonArray questionJsonArray =
                        JsonParser.parseString(json).getAsJsonObject()
                                .getAsJsonObject("data")
                                .getAsJsonObject("problemsetQuestionList")
                                .getAsJsonArray("questions");

                for (JsonElement jsonElement : questionJsonArray) {
                    questionNames.add(jsonElement.getAsJsonObject().get("titleSlug").getAsString());
                }

                System.out.println("Got result " + questionJsonArray.size());
            }

            System.out.println("Total result: " + questionNames.size());

            // Fetch all question details
            QuestionDetail detail = null;
            for (String questionTitleSlug : questionNames) {
                System.out.println("Getting question: " + questionTitleSlug);
                json = NetworkUtil.post(API_BASE, String.format(DETAILS_REQUEST_BODY, questionTitleSlug), headers);

                JsonObject question =
                        JsonParser.parseString(json).getAsJsonObject()
                                .getAsJsonObject("data")
                                .getAsJsonObject("question");

                detail = new Gson().fromJson(question, QuestionDetail.class);
                record.writeToFile(
                        "D:\\ojs-leetcode",
                        String.valueOf(detail.getId()),
                        detail.getChineseTitle() == null ? detail.getTitle() : detail.getChineseTitle(),
                        record.fillText(detail));

                Thread.sleep(random.getRandomNumber(120));
            }

        } catch (IOException | InterruptedException e) {
            throw new RuntimeException(e);
        }

        System.out.println("Finish.");
        return QuestionDetails;
    }
}