package net.lensfrex.oj.collector;

import net.lensfrex.oj.collector.data.QuestionDetail;
import net.lensfrex.oj.collector.data.Tag;
import net.lensfrex.oj.collector.utils.IOUtil;
import net.lensfrex.oj.collector.utils.ParamFiller;

import java.io.File;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.List;
import java.util.regex.Pattern;

public class Record {
    private static final String MAIN_TEMPLATE = IOUtil.inputStreamToString(Record.class.getResourceAsStream("/template.txt"), StandardCharsets.UTF_8);

    public String fillText(QuestionDetail results) {
        System.out.println("Filling: " + results.getChineseTitle());
        HashMap<String, String> params = new HashMap<>();

        if (results.isPaidOnly()) {
            params.put("englishContent", "这道题是~付~费~内容哦");
        } else {
            params.put("englishContent", results.getContent());
        }

        params.put("englishTitle", results.getTitle());
        params.put("title", results.getChineseTitle());
        params.put("content", results.getChineseContent());
        params.put("level", results.getDifficulty());
        params.put("id", results.getId());

        StringBuilder tagStringBuilder = new StringBuilder();
        List<Tag> tags = results.getTags();
        for (Tag tag : tags) {
            tagStringBuilder.append('[').append(tag.getChineseName()).append("] ");
        }

        params.put("tags", tagStringBuilder.toString());

        StringBuilder hintStringBuilder = new StringBuilder();
        List<String> hints = results.getHints();
        for (String hint : hints) {
            hintStringBuilder.append(hint).append("\n\n");
        }

        params.put("hints", hintStringBuilder.toString());

        return ParamFiller.fill(MAIN_TEMPLATE, params);
    }

    private static final Pattern FilePattern = Pattern.compile("[\\\\/:*?\"<>|]");

    private static String filenameFilter(String str) {
        return str == null ? null : FilePattern.matcher(str).replaceAll("");
    }

    public void writeToFile(String location, String id, String title, String context) {
        id = filenameFilter(id);
        title = filenameFilter(title);

        File file = new File(String.format("%s/%s - %s.md", location, id, title));
        System.out.println("Will write to: " + file.getPath());
        try {
            IOUtil.writeFile(context.getBytes(StandardCharsets.UTF_8), file);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
}
